@echo off

set ARGS=%*
set ADBPATH=bin\adb
set TMPFILE=tmp\tmp

REM ## initialize
cd "%~dp0"
if not exist tmp mkdir tmp
if exist %TMPFILE% del %TMPFILE%
title FRep2 grant screen capture permission
echo [FRep2 grant capture permission]
REM ## terminate running adb process first
%ADBPATH% kill-server > NUL 2>&1
%ADBPATH% start-server > NUL
echo.

REM ## check Android version and USB debug connection
:CHECKOSVER
set ANDROIDVER=0
setlocal ENABLEDELAYEDEXPANSION
set /A SDKNUM=0
for /F "usebackq tokens=*" %%i in (`%ADBPATH% %ARGS% shell getprop ro.build.version.sdk 2^>^&1`) do (
  if "%%i"=="adb.exe: more than one device/emulator" (
    goto MULTI
  ) else if "%%i"=="adb.exe: no devices/emulators found" (
    echo error: device not found.
    echo.
    goto FAILED
  ) else if "%%i"=="error: device offline" (
    goto OFFLINE
  ) else (
REM ## for matching; find returns ERRORLEVEL 0 for success
    echo %%i | find "device unauthorized" > NUL
    if not ERRORLEVEL 1 (
      goto OFFLINE
    )
REM ## skip version check on permission denied / not found
    echo %%i | find "permission denied" > NUL
    if not ERRORLEVEL 1 (
      goto START
    )
    echo %%i | find "not found" > NUL
    if not ERRORLEVEL 1 (
      goto START
    )
REM ## check SDK version
    set /A SDKNUM=%%i+0
    if "!SDKNUM!"=="0" (
REM ## not a number / unknown error.
      goto FAILED
    )
    goto START
  )
)

:START
endlocal & set ANDROIDVER=%SDKNUM%

REM ## not work on Android ~6
if %ANDROIDVER% leq 23 goto FAILUNDER

%ADBPATH% %ARGS% shell cmd appops set com.x0.strai.secondfrep PROJECT_MEDIA allow

REM ## to remove the permission, use following
REM ## %ADBPATH% %ARGS% shell cmd appops set com.x0.strai.secondfrep PROJECT_MEDIA ignore

if ERRORLEVEL 1 goto :FAILED

echo Granted screen capture permission to FRep2.
echo.
%ADBPATH% kill-server
pause
exit /b


:OFFLINE
echo.
set /P TMP="Allow USB debugging on Android device and then push Enter key. (Ctrl-C to cancel) "
goto CHECKOSVER


:MULTI
setlocal ENABLEDELAYEDEXPANSION
REM # requires for counting in loop
set /A DEVNUM=0
echo Multiple Android devices found.
for /F "usebackq tokens=*" %%i in (`%ADBPATH% devices -l`) do (
  if not "%%i"=="" (
REM # ignore header; find returns ERRORLEVEL=0 for success
    echo %%i | find "List of devices attached" > NUL
    if ERRORLEVEL 1 (
      CALL :ECHODEVICE !DEVNUM! %%i
    )
    set /A DEVNUM=!DEVNUM!+1
  )
)
echo.
set /P SELNUM="Enter the number of device to setup FRep2. (0 to cancel):"
REM set zero for non-numeric value
set /A SELNUM=%SELNUM%+0
set /A DEVNUM=0
set SELDEVICE=
set SAMPLEDEVICE=00000000

if "0"=="%SELNUM%" (
  echo Setup cancelled.
  echo.
  pause
  %ADBPATH% kill-server
  exit /b 1
)

for /F "usebackq tokens=1" %%i in (`%ADBPATH% devices`) do (
  if not "%%i"=="" (
REM # ignore header; find returns ERRORLEVEL=0 for success
    echo %%i | find "List" > NUL
    if ERRORLEVEL 1 (
	  if "!DEVNUM!"=="%SELNUM%" (
        set SELDEVICE=%%i
		goto SELECTED
	  ) else if "%SAMPLEDEVICE%"=="00000000" (
	    set SAMPLEDEVICE=%%i
	  )
    )
    set /A DEVNUM=!DEVNUM!+1
  )
)
:SELECTED
if "%SELDEVICE%"=="" (
  echo Retry with only single Android device connection, or designate
  echo single Android device by option -s ^(single serial from above^)
  echo ex.^) frep2 -s %SAMPLEDEVICE%
  echo.
  pause
  %ADBPATH% kill-server
  exit /b 1
)
echo ^(%SELNUM%^) %SELDEVICE% is selected.
endlocal & set ARGS=-s %SELDEVICE%
goto CHECKOSVER

:FAILED
echo Failed to grant permission to FRep2. Please check followings.
echo - Confirm FRep2 app is installed.
echo - Confirm Android Settings - Developer options - USB Debugging: ON.
echo - Connect Android Device with USB cable for communication.
echo See also support site. http://strai.x0.com/frep2/
echo.
pause
%ADBPATH% kill-server
exit /b

:FAILUNDER
echo FRep2 is available only on Android 7~.
echo For Android ~6, please consider previous edition.
echo [FRep] http://strai.x0.com/frep/
echo.
pause
%ADBPATH% kill-server
exit /b

REM ## dump device item; SERIALID, (from product before transport_id) (separated because of for)
:ECHODEVICE
SETLOCAL
set NUM=%1
set SERIAL=%2
set DEVICE=
shift
shift
:DEVICECOL
if "%1"=="" (
  echo ^(%NUM%^) %SERIAL%%DEVICE%
  exit /b
)
set X=%1
set BASE=%DEVICE%
if "device"=="%X%" (
  REM ignore device keyword
) else if "product:"=="%X:~0,8%" (
  set DEVICE=%BASE% %X:~8%
) else if "transport"=="%X:~0,9%" (
  echo ^(%NUM%^) %SERIAL%%DEVICE%
  exit /b
) else (
  set DEVICE=%BASE% %X%
)
shift
goto :DEVICECOL

REM ## usage: call :STRLEN (quoted string)
REM ##        call :STRLEN "STRING", call :STRLEN "%VAR%", call :STRLEN %VARQUOTED%
REM ##    set /a LENGTH=%ERRORLEVEL%
:STRLEN
SETLOCAL
set STR=%~1
if "%STR%"=="" exit /b 0
set /a LEN=1
:STRLOOP
set STR=%STR:~1%
if "%STR%"=="" exit /b %LEN%
set /a LEN=%LEN%+1
goto :STRLOOP
